function outputdata = MLSM()
	tic
	clc
	close all
	clear all
    clf
    warning off
    
	fprintf('Starting Particle Swarm Optimisation using MLSM program..\n');
    global modelInfo
    global mlsmData
	global doeData
	global mlsmValidationData
	global mlsmOutputData
	
    % optimise analytical (0) or metamodel-based (1) response surface
    modelInfo.using_metamodel=1;
    
    % read in DoE and design space data
    % create responses at DoE points using analytical solution
    modelInfo.using_metamodel=0;    
    doeData=readDoEData();

    % intialise DoE data
    % do optimisation using metamodel responses
    modelInfo.using_metamodel=1;

    x=doeData.x; n=doeData.n; ndv=doeData.ndv; rang=doeData.rang; f=doeData.f;
    bounds=doeData.bounds;

    % split up read so that each bit of information is read in separately
	% read in MLSM parameters
	mlsmData=readMLSMData();
    
    % initialise mlsm data
    glb=mlsmData.glb; gub=mlsmData.gub; gtol=mlsmData.gtol; giter=mlsmData.giter;
    npoly=mlsmData.npoly; sens=mlsmData.sens; s = mlsmData.s; vmethod=mlsmData.vmethod;
    dWmin=mlsmData.dWmin; nExcessPts=mlsmData.nExcessPts; gopt=mlsmData.gopt;
    thetarange=mlsmData.thetarange; plotrange=mlsmData.plotrange; 

    % read in MLSM validation data
	mlsmValidationData=readMLSMValidationData();

    % initialise validation data
    xv=mlsmValidationData.xv; nxv=mlsmValidationData.nxv;
    fxv=mlsmValidationData.fxv;

    % read in output point data
	mlsmOutputData=readOutputData();
	
    outputdata = [];
    
	% optimise theta for MLSM (gopt=1 by default)
    
    % Run golden search optimisation to find optimal theta value
	[theta,RMSE]=golden(glb,gub,gtol,giter,x,xv,n,nxv,npoly,ndv,f,fxv,sens,rang,s,vmethod,dWmin,nExcessPts);
    mlsmData.theta=theta;
		
    % initalise plan points where MLSM response to be calculated
	mlsmOutputData=readOutputData();
    xa=mlsmOutputData.xa; na=mlsmOutputData.na;
    
	% Evaluate plan points xa using MLSM with optimised theta value
	fprintf('Creating approximation using theta = %f\n', theta);
	ya =movingls(theta,n,na,x,xa,npoly,ndv,f,sens,rang,s,dWmin,nExcessPts);
	save function_approx_optimum.txt ya
	fprintf('Finished creating approximation..\n');
         
    %Code for visualising predicted surface
    xz = [];
    for i = 1:41
        for j = 1:41
            xz(j,i) = ya(((i-1)*41)+j);
        end
    end

    close all
        
    [xcd ycd] = meshgrid(0:1/40:1,0:1/40:1);
        
    %Display the predicted surface
    string1 = 'MLS Hump Camel Back Function with OLH sampling n= ';
    string2 = ' nv = ';
	string3 = ' optimum theta=';
    titlestring = strcat(string1,num2str(n),string2,num2str(nxv),...
        string3,num2str(theta));

    figure(1)
    surf(xcd,ycd,xz)

    xlabel('x1')
    ylabel('x2')
    zlabel('f(x1,x2)')
    title(titlestring)
    hold on
        	
	% plot MLS response at DoE points on the MLS response surface
	ydoe =movingls(theta,n,n,x,x,npoly,ndv,f,sens,rang,s,dWmin,nExcessPts);
    scatter3(x(:,1),x(:,2),ydoe,15,'red','fill');
      
	%PLOTTING
	if plotrange==1
		fprintf('Plotting results..\n');
		for i=1:size(ya,2)
			figure(i+1);
 			plotfunction(100,bounds)
            scatter3(xa(:,1),xa(:,2),ya,100,'xr');
 			axis([bounds(1,:), bounds(2,:)])
            
            xlabel('x1');
            ylabel('x2');
            zlabel('F(x1,x2)');
		end
		fprintf('Finished plotting results..\n');
     end
          
 
    PS=1;   % run Particle Swarm to find global minimum of response function
    if PS==1
        psoOptions = get_psoOptions;
        
        % update PS parameters to comply with those already set for design space 
        % note have to be careful with design variable boundaries as these
        % are currently assumed to be the same in each variable - would
        % need to update PS code to allow this (quite straightforward)
        psoOptions.SParams.lb = bounds(1,1);
        psoOptions.Sparams.ub = bounds(1,2);
        psoOptions.Vars.Dim = ndv;
        
        [fxmin, xmin, Swarm, history] = pso(psoOptions);
        x1opt = xmin(1,1);
        x2opt = xmin(1,2);
        Optval = fxmin;
        fprintf('Optimum design variables: x1opt=%f, x2opt=%f\n',x1opt,x2opt);
        fprintf('Optval=%f\n',fxmin);
        fprintf('True prediction at optimum point=%f\n',analytical_response([x1opt x2opt]))
        scatter3(x1opt,x2opt,Optval,50,'yellow','fill');
    end

	fprintf('Finished PSO MLSM program..\n');
	toc
end